@@ -1,4 +1,4 @@
-/*	$OpenBSD: output.c,v 1.41 2023/04/28 13:24:25 claudio Exp $ */
+/*	$OpenBSD: output.c,v 1.50 2024/01/31 11:23:20 claudio Exp $ */
 
 /*
  * Copyright (c) 2003 Henning Brauer <henning@openbsd.org>
@@ -388,10 +388,6 @@ show_neighbor_full(struct peer *p, struct parse_result *res)
 
 	show_neighbor_msgstats(p);
 	printf("\n");
-	if (p->stats.last_reason[0]) {
-		printf("  Last received shutdown reason: \"%s\"\n",
-		    log_reason(p->stats.last_reason));
-	}
 
 	errstr = fmt_errstr(p->stats.last_sent_errcode,
 	    p->stats.last_sent_suberr);
@@ -401,6 +397,10 @@ show_neighbor_full(struct peer *p, struct parse_result *res)
 	    p->stats.last_rcvd_suberr);
 	if (errstr)
 		printf("  Last error received: %s\n", errstr);
+	if (p->stats.last_reason[0]) {
+		printf("  Last received shutdown reason: \"%s\"\n",
+		    log_reason(p->stats.last_reason));
+	}
 
 	if (p->state >= STATE_OPENSENT) {
 		printf("  Local host:  %20s, Local port:  %5u\n",
@@ -646,18 +646,17 @@ show_interface(struct ctl_show_interface *iface)
 }
 
 static void
-show_communities(u_char *data, size_t len, struct parse_result *res)
+show_communities(struct ibuf *data, struct parse_result *res)
 {
 	struct community c;
-	size_t	i;
 	uint64_t ext;
 	uint8_t type = 0;
 
-	if (len % sizeof(c))
-		return;
-
-	for (i = 0; i < len; i += sizeof(c)) {
-		memcpy(&c, data + i, sizeof(c));
+	while (ibuf_size(data) != 0) {
+		if (ibuf_get(data, &c, sizeof(c)) == -1) {
+			warn("communities");
+			break;
+		}
 
 		if (type != c.flags) {
 			if (type != 0)
@@ -690,9 +689,7 @@ show_communities(u_char *data, size_t len, struct parse_result *res)
 				ext |= (uint64_t)c.data2 & 0xffff;
 				break;
 			}
-			ext = htobe64(ext);
-
-			printf(" %s", fmt_ext_community((void *)&ext));
+			printf(" %s", fmt_ext_community(ext));
 			break;
 		}
 	}
@@ -701,207 +698,167 @@ show_communities(u_char *data, size_t len, struct parse_result *res)
 }
 
 static void
-show_community(u_char *data, uint16_t len)
+show_community(struct ibuf *buf)
 {
 	uint16_t	a, v;
-	uint16_t	i;
 
-	if (len & 0x3) {
-		printf("bad length");
-		return;
-	}
-
-	for (i = 0; i < len; i += 4) {
-		memcpy(&a, data + i, sizeof(a));
-		memcpy(&v, data + i + 2, sizeof(v));
-		a = ntohs(a);
-		v = ntohs(v);
+	while (ibuf_size(buf) > 0) {
+		if (ibuf_get_n16(buf, &a) == -1 ||
+		    ibuf_get_n16(buf, &v) == -1) {
+			printf("bad length");
+			return;
+		}
 		printf("%s", fmt_community(a, v));
 
-		if (i + 4 < len)
+		if (ibuf_size(buf) > 0)
 			printf(" ");
 	}
 }
 
 static void
-show_large_community(u_char *data, uint16_t len)
+show_large_community(struct ibuf *buf)
 {
 	uint32_t	a, l1, l2;
-	uint16_t	i;
 
-	if (len % 12) {
-		printf("bad length");
-		return;
-	}
-
-	for (i = 0; i < len; i += 12) {
-		memcpy(&a, data + i, sizeof(a));
-		memcpy(&l1, data + i + 4, sizeof(l1));
-		memcpy(&l2, data + i + 8, sizeof(l2));
-		a = ntohl(a);
-		l1 = ntohl(l1);
-		l2 = ntohl(l2);
+	while (ibuf_size(buf) > 0) {
+		if (ibuf_get_n32(buf, &a) == -1 ||
+		    ibuf_get_n32(buf, &l1) == -1 ||
+		    ibuf_get_n32(buf, &l2) == -1) {
+			printf("bad length");
+			return;
+		}
 		printf("%s", fmt_large_community(a, l1, l2));
 
-		if (i + 12 < len)
+		if (ibuf_size(buf) > 0)
 			printf(" ");
 	}
 }
 
 static void
-show_ext_community(u_char *data, uint16_t len)
+show_ext_community(struct ibuf *buf)
 {
-	uint16_t	i;
+	uint64_t	ext;
 
-	if (len & 0x7) {
-		printf("bad length");
-		return;
-	}
+	while (ibuf_size(buf) > 0) {
+		if (ibuf_get_n64(buf, &ext) == -1) {
+			printf("bad length");
+			return;
+		}
+		printf("%s", fmt_ext_community(ext));
 
-	for (i = 0; i < len; i += 8) {
-		printf("%s", fmt_ext_community(data + i));
-
-		if (i + 8 < len)
+		if (ibuf_size(buf) > 0)
 			printf(" ");
 	}
 }
 
 static void
-show_attr(u_char *data, size_t len, int reqflags, int addpath)
+show_attr(struct ibuf *buf, int reqflags, int addpath)
 {
-	u_char		*path;
 	struct in_addr	 id;
 	struct bgpd_addr prefix;
+	struct ibuf	 asbuf, *path = NULL;
 	char		*aspath;
-	uint32_t	 as, pathid;
-	uint16_t	 alen, ioff, short_as, afi;
-	uint8_t		 flags, type, safi, aid, prefixlen;
-	int		 i, pos, e2, e4;
+	size_t		 i, alen;
+	uint32_t	 as, pathid, val;
+	uint16_t	 short_as, afi;
+	uint8_t		 flags, type, safi, aid, prefixlen, origin, b;
+	int		 e2, e4;
 
-	if (len < 3) {
-		warnx("Too short BGP attribute");
-		return;
-	}
-
-	flags = data[0];
-	type = data[1];
+	if (ibuf_get_n8(buf, &flags) == -1 ||
+	    ibuf_get_n8(buf, &type) == -1)
+		goto bad_len;
 
 	/* get the attribute length */
 	if (flags & ATTR_EXTLEN) {
-		if (len < 4) {
-			warnx("Too short BGP attribute");
-			return;
-		}
-		memcpy(&alen, data+2, sizeof(uint16_t));
-		alen = ntohs(alen);
-		data += 4;
-		len -= 4;
+		uint16_t attr_len;
+		if (ibuf_get_n16(buf, &attr_len) == -1)
+			goto bad_len;
+		alen = attr_len;
 	} else {
-		alen = data[2];
-		data += 3;
-		len -= 3;
+		uint8_t attr_len;
+		if (ibuf_get_n8(buf, &attr_len) == -1)
+			goto bad_len;
+		alen = attr_len;
 	}
 
 	/* bad imsg len how can that happen!? */
-	if (alen > len) {
-		warnx("Bad BGP attribute length");
-		return;
-	}
+	if (alen > ibuf_size(buf))
+		goto bad_len;
 
 	printf("    %s: ", fmt_attr(type, flags));
 
 	switch (type) {
 	case ATTR_ORIGIN:
-		if (alen == 1)
-			printf("%s", fmt_origin(*data, 0));
-		else
-			printf("bad length");
+		if (alen != 1 || ibuf_get_n8(buf, &origin) == -1)
+			goto bad_len;
+		printf("%s", fmt_origin(origin, 0));
 		break;
 	case ATTR_ASPATH:
 	case ATTR_AS4_PATH:
 		/* prefer 4-byte AS here */
-		e4 = aspath_verify(data, alen, 1, 0);
-		e2 = aspath_verify(data, alen, 0, 0);
+		e4 = aspath_verify(buf, 1, 0);
+		e2 = aspath_verify(buf, 0, 0);
 		if (e4 == 0 || e4 == AS_ERR_SOFT) {
-			path = data;
+			ibuf_from_ibuf(&asbuf, buf);
 		} else if (e2 == 0 || e2 == AS_ERR_SOFT) {
-			path = aspath_inflate(data, alen, &alen);
-			if (path == NULL)
-				errx(1, "aspath_inflate failed");
+			if ((path = aspath_inflate(buf)) == NULL) {
+				printf("aspath_inflate failed");
+				break;
+			}
+			ibuf_from_ibuf(&asbuf, path);
 		} else {
 			printf("bad AS-Path");
 			break;
 		}
-		if (aspath_asprint(&aspath, path, alen) == -1)
+		if (aspath_asprint(&aspath, &asbuf) == -1)
 			err(1, NULL);
 		printf("%s", aspath);
 		free(aspath);
-		if (path != data)
-			free(path);
+		ibuf_free(path);
 		break;
 	case ATTR_NEXTHOP:
-		if (alen == 4) {
-			memcpy(&id, data, sizeof(id));
-			printf("%s", inet_ntoa(id));
-		} else
-			printf("bad length");
+	case ATTR_ORIGINATOR_ID:
+		if (alen != 4 || ibuf_get(buf, &id, sizeof(id)) == -1)
+			goto bad_len;
+		printf("%s", inet_ntoa(id));
 		break;
 	case ATTR_MED:
 	case ATTR_LOCALPREF:
-		if (alen == 4) {
-			uint32_t val;
-			memcpy(&val, data, sizeof(val));
-			val = ntohl(val);
-			printf("%u", val);
-		} else
-			printf("bad length");
+		if (alen != 4 || ibuf_get_n32(buf, &val) == -1)
+			goto bad_len;
+		printf("%u", val);
 		break;
 	case ATTR_AGGREGATOR:
 	case ATTR_AS4_AGGREGATOR:
 		if (alen == 8) {
-			memcpy(&as, data, sizeof(as));
-			memcpy(&id, data + sizeof(as), sizeof(id));
-			as = ntohl(as);
+			if (ibuf_get_n32(buf, &as) == -1 ||
+			    ibuf_get(buf, &id, sizeof(id)) == -1)
+				goto bad_len;
 		} else if (alen == 6) {
-			memcpy(&short_as, data, sizeof(short_as));
-			memcpy(&id, data + sizeof(short_as), sizeof(id));
-			as = ntohs(short_as);
+			if (ibuf_get_n16(buf, &short_as) == -1 ||
+			    ibuf_get(buf, &id, sizeof(id)) == -1)
+				goto bad_len;
+			as = short_as;
 		} else {
-			printf("bad length");
-			break;
+			goto bad_len;
 		}
 		printf("%s [%s]", log_as(as), inet_ntoa(id));
 		break;
 	case ATTR_COMMUNITIES:
-		show_community(data, alen);
-		break;
-	case ATTR_ORIGINATOR_ID:
-		if (alen == 4) {
-			memcpy(&id, data, sizeof(id));
-			printf("%s", inet_ntoa(id));
-		} else
-			printf("bad length");
+		show_community(buf);
 		break;
 	case ATTR_CLUSTER_LIST:
-		for (ioff = 0; ioff + sizeof(id) <= alen;
-		    ioff += sizeof(id)) {
-			memcpy(&id, data + ioff, sizeof(id));
+		while (ibuf_size(buf) > 0) {
+			if (ibuf_get(buf, &id, sizeof(id)) == -1)
+				goto bad_len;
 			printf(" %s", inet_ntoa(id));
 		}
 		break;
 	case ATTR_MP_REACH_NLRI:
 	case ATTR_MP_UNREACH_NLRI:
-		if (alen < 3) {
- bad_len:
-			printf("bad length");
-			break;
-		}
-		memcpy(&afi, data, 2);
-		data += 2;
-		alen -= 2;
-		afi = ntohs(afi);
-		safi = *data++;
-		alen--;
+		if (ibuf_get_n16(buf, &afi) == -1 ||
+		    ibuf_get_n8(buf, &safi) == -1)
+			goto bad_len;
 
 		if (afi2aid(afi, safi, &aid) == -1) {
 			printf("bad AFI/SAFI pair");
@@ -912,11 +869,7 @@ show_attr(u_char *data, size_t len, int reqflags, int addpath)
 		if (type == ATTR_MP_REACH_NLRI) {
 			struct bgpd_addr nexthop;
 			uint8_t nhlen;
-			if (len == 0)
-				goto bad_len;
-			nhlen = *data++;
-			alen--;
-			if (nhlen > len)
+			if (ibuf_get_n8(buf, &nhlen) == -1)
 				goto bad_len;
 			memset(&nexthop, 0, sizeof(nexthop));
 			switch (aid) {
@@ -924,103 +877,103 @@ show_attr(u_char *data, size_t len, int reqflags, int addpath)
 				nexthop.aid = aid;
 				if (nhlen != 16 && nhlen != 32)
 					goto bad_len;
-				memcpy(&nexthop.v6.s6_addr, data, 16);
+				if (ibuf_get(buf, &nexthop.v6,
+				    sizeof(nexthop.v6)) == -1)
+					goto bad_len;
 				break;
 			case AID_VPN_IPv4:
 				if (nhlen != 12)
 					goto bad_len;
 				nexthop.aid = AID_INET;
-				memcpy(&nexthop.v4, data + sizeof(uint64_t),
-				    sizeof(nexthop.v4));
+				if (ibuf_skip(buf, sizeof(uint64_t)) == -1 ||
+				    ibuf_get(buf, &nexthop.v4,
+				    sizeof(nexthop.v4)) == -1)
+					goto bad_len;
 				break;
 			case AID_VPN_IPv6:
 				if (nhlen != 24)
 					goto bad_len;
 				nexthop.aid = AID_INET6;
-				memcpy(&nexthop.v6, data + sizeof(uint64_t),
-				    sizeof(nexthop.v6));
+				if (ibuf_skip(buf, sizeof(uint64_t)) == -1 ||
+				    ibuf_get(buf, &nexthop.v6,
+				    sizeof(nexthop.v6)) == -1)
+					goto bad_len;
 				break;
 			default:
 				printf("unhandled AID #%u", aid);
 				goto done;
 			}
 			/* ignore reserved (old SNPA) field as per RFC4760 */
-			data += nhlen + 1;
-			alen -= nhlen + 1;
+			if (ibuf_skip(buf, 1) == -1)
+				goto bad_len;
 
 			printf(" nexthop: %s", log_addr(&nexthop));
 		}
 
-		while (alen > 0) {
-			if (addpath) {
-				if (alen <= sizeof(pathid)) {
-					printf("bad nlri prefix");
-					return;
-				}
-				memcpy(&pathid, data, sizeof(pathid));
-				pathid = ntohl(pathid);
-				data += sizeof(pathid);
-				alen -= sizeof(pathid);
-			}
+		while (ibuf_size(buf) > 0) {
+			if (addpath)
+				if (ibuf_get_n32(buf, &pathid) == -1)
+					goto bad_len;
 			switch (aid) {
 			case AID_INET6:
-				pos = nlri_get_prefix6(data, alen, &prefix,
-				    &prefixlen);
+				if (nlri_get_prefix6(buf, &prefix,
+				    &prefixlen) == -1)
+					goto bad_len;
 				break;
 			case AID_VPN_IPv4:
-				pos = nlri_get_vpn4(data, alen, &prefix,
-				    &prefixlen, 1);
+				if (nlri_get_vpn4(buf, &prefix,
+				    &prefixlen, 1) == -1)
+					goto bad_len;
 				break;
 			case AID_VPN_IPv6:
-				pos = nlri_get_vpn6(data, alen, &prefix,
-				    &prefixlen, 1);
+				if (nlri_get_vpn6(buf, &prefix,
+				    &prefixlen, 1) == -1)
+					goto bad_len;
 				break;
 			default:
 				printf("unhandled AID #%u", aid);
 				goto done;
 			}
-			if (pos == -1) {
-				printf("bad %s prefix", aid2str(aid));
-				break;
-			}
 			printf(" %s/%u", log_addr(&prefix), prefixlen);
 			if (addpath)
 				printf(" path-id %u", pathid);
-			data += pos;
-			alen -= pos;
 		}
 		break;
 	case ATTR_EXT_COMMUNITIES:
-		show_ext_community(data, alen);
+		show_ext_community(buf);
 		break;
 	case ATTR_LARGE_COMMUNITIES:
-		show_large_community(data, alen);
+		show_large_community(buf);
 		break;
 	case ATTR_OTC:
-		if (alen == 4) {
-			memcpy(&as, data, sizeof(as));
-			as = ntohl(as);
-			printf("%s", log_as(as));
-		} else {
-			printf("bad length");
-		}
+		if (alen != 4 || ibuf_get_n32(buf, &as) == -1)
+			goto bad_len;
+		printf("%s", log_as(as));
 		break;
 	case ATTR_ATOMIC_AGGREGATE:
 	default:
-		printf(" len %u", alen);
+		printf(" len %zu", alen);
 		if (alen) {
 			printf(":");
-			for (i=0; i < alen; i++)
-				printf(" %02x", *(data+i));
+			for (i = 0; i < alen; i++) {
+				if (ibuf_get_n8(buf, &b) == -1)
+					goto bad_len;
+				printf(" %02x", b);
+			}
 		}
 		break;
 	}
+
  done:
 	printf("%c", EOL0(reqflags));
+	return;
+
+ bad_len:
+	printf("bad length%c", EOL0(reqflags));
 }
 
 static void
-show_rib_brief(struct ctl_show_rib *r, u_char *asdata, size_t aslen)
+show_rib_brief(struct ctl_show_rib *r, struct ibuf *asbuf)
 {
 	char *p, *aspath;
 
@@ -1032,7 +985,7 @@ show_rib_brief(struct ctl_show_rib *r, u_char *asdata, size_t aslen)
 	    log_addr(&r->exit_nexthop), r->local_pref, r->med);
 	free(p);
 
-	if (aspath_asprint(&aspath, asdata, aslen) == -1)
+	if (aspath_asprint(&aspath, asbuf) == -1)
 		err(1, NULL);
 	if (strlen(aspath) > 0)
 		printf("%s ", aspath);
@@ -1042,8 +995,7 @@ show_rib_brief(struct ctl_show_rib *r, u_char *asdata, size_t aslen)
 }
 
 static void
-show_rib_detail(struct ctl_show_rib *r, u_char *asdata, size_t aslen,
-    int flag0)
+show_rib_detail(struct ctl_show_rib *r, struct ibuf *asbuf, int flag0)
 {
 	struct in_addr		 id;
 	char			*aspath, *s;
@@ -1052,7 +1004,7 @@ show_rib_detail(struct ctl_show_rib *r, u_char *asdata, size_t aslen,
 	    log_addr(&r->prefix), r->prefixlen,
 	    EOL0(flag0));
 
-	if (aspath_asprint(&aspath, asdata, aslen) == -1)
+	if (aspath_asprint(&aspath, asbuf) == -1)
 		err(1, NULL);
 	if (strlen(aspath) > 0)
 		printf("    %s%c", aspath, EOL0(flag0));
@@ -1079,13 +1031,12 @@ show_rib_detail(struct ctl_show_rib *r, u_char *asdata, size_t aslen,
 }
 
 static void
-show_rib(struct ctl_show_rib *r, u_char *asdata, size_t aslen,
-    struct parse_result *res)
+show_rib(struct ctl_show_rib *r, struct ibuf *aspath, struct parse_result *res)
 {
 	if (res->flags & F_CTL_DETAIL)
-		show_rib_detail(r, asdata, aslen, res->flags);
+		show_rib_detail(r, aspath, res->flags);
 	else
-		show_rib_brief(r, asdata, aslen);
+		show_rib_brief(r, aspath);
 }
 
 static void
@@ -1171,28 +1122,29 @@ show_rtr(struct ctl_show_rtr *rtr)
 
 	printf("RTR neighbor is %s, port %u\n",
 	    log_addr(&rtr->remote_addr), rtr->remote_port);
+	printf(" State: %s\n", rtr->state);
 	if (rtr->descr[0])
 		printf(" Description: %s\n", rtr->descr);
 	if (rtr->local_addr.aid != AID_UNSPEC)
 		printf(" Local Address: %s\n", log_addr(&rtr->local_addr));
 	if (rtr->session_id != -1)
-		printf (" Session ID: %d Serial #: %u\n",
-		    rtr->session_id, rtr->serial);
+		printf(" Version: %u Session ID: %d Serial #: %u\n",
+		    rtr->version, rtr->session_id, rtr->serial);
 	printf(" Refresh: %u, Retry: %u, Expire: %u\n",
 	    rtr->refresh, rtr->retry, rtr->expire);
 
 	if (rtr->last_sent_error != NO_ERROR) {
 		printf(" Last sent error: %s\n",
-		  log_rtr_error(rtr->last_sent_error));
+		    log_rtr_error(rtr->last_sent_error));
 		if (rtr->last_sent_msg[0])
-			printf("   with reason \"%s\"",
+			printf("   with reason \"%s\"\n",
 			    log_reason(rtr->last_sent_msg));
 	}
 	if (rtr->last_recv_error != NO_ERROR) {
 		printf(" Last received error: %s\n",
-		  log_rtr_error(rtr->last_recv_error));
+		    log_rtr_error(rtr->last_recv_error));
 		if (rtr->last_recv_msg[0])
-			printf("   with reason \"%s\"",
+			printf("   with reason \"%s\"\n",
 			    log_reason(rtr->last_recv_msg));
 	}
 
